//
//  AppDelegate.m
//  CoreDataLibraryAppOSX
//
//  Created by Duncan Groenewald on 4/02/2014.
//  Copyright (c) 2014 OSSH Pty Ltd. All rights reserved.
//

#import "AppDelegate.h"

@implementation AppDelegate

//@synthesize persistentStoreCoordinator = _persistentStoreCoordinator;
//@synthesize managedObjectModel = _managedObjectModel;
//@synthesize managedObjectContext = _managedObjectContext;

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    // Insert code here to initialize your application
    [[OSxCDStackManager sharedManager] setUbiquityContainerKey:UBIQUITY_CONTAINER_KEY];
    [[OSxCDStackManager sharedManager] setUbiquityIDToken:UBID_TOKEN];
    [[OSxCDStackManager sharedManager] setCloudPreferenceKey:COMPANY_ID "." APP_NAME "." CLOUD_PREFERENCE_KEY];
    [[OSxCDStackManager sharedManager] setCloudPreferenceSet:COMPANY_ID "." APP_NAME "." CLOUD_PREFERENCE_SET];
    [[OSxCDStackManager sharedManager] setMakeBackupPreferenceKey:COMPANY_ID "." APP_NAME "." BACKUP_PREFERENCE_KEY];
    [[OSxCDStackManager sharedManager] setLoadInBackgroundPreferenceKey:COMPANY_ID "." APP_NAME "." BACKGROUNDLOAD_PREFERENCE_KEY];
    
    mainMessage.stringValue = @"Loading...";
    
    //Register for StoreReady notification
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(storeReady) name:OSStoreReadyNotification object:[OSxCDStackManager sharedManager]];
    
    [[OSxCDStackManager sharedManager] checkUserICloudPreferenceAndSetupIfNecessary];
    
}
- (void)storeReady {
    FLOG(@" called");
    [self performSelectorOnMainThread:@selector(setContentViewToMain) withObject:nil waitUntilDone:NO];
}
- (void)setContentViewToMainX  {
    FLOG(@" called");
}
- (void)setContentViewToMain  {
   
    FLOG(@" called");
    if (_currentContentViewController) {
        
        [[_currentContentViewController view] removeFromSuperview];
        _currentContentViewController = nil;
    }
    _currentContentViewController = [self createMainView]; // Retained
    NSView *view = [_currentContentViewController view];
    view.frame = _mainContentView.bounds;
    [view setAutoresizingMask:NSViewWidthSizable | NSViewHeightSizable];
    [_mainContentView addSubview:view];
}
- (NSViewController*)createMainView
{
    FLOG(@" called");
    _mainViewController = nil;
    if (_mainViewController == nil) {
        
        _mainViewController = [[MainViewController alloc] initWithNibName:@"MainViewController" bundle:nil];
        _mainViewController.managedObjectContext = [[OSxCDStackManager sharedManager] managedObjectContext];
        FLOG(@" _mainViewController.managedObjectContext = %@", _mainViewController.managedObjectContext);

        
    }
    return _mainViewController;
}
// Returns the managed object context for the application (which is already bound to the persistent store coordinator for the application.) 
- (NSManagedObjectContext *)managedObjectContext
{

    return [[OSxCDStackManager sharedManager] managedObjectContext];
}

// Returns the NSUndoManager for the application. In this case, the manager returned is that of the managed object context for the application.
- (NSUndoManager *)windowWillReturnUndoManager:(NSWindow *)window
{
    return [[[OSxCDStackManager sharedManager] managedObjectContext] undoManager];
}

// Performs the save action for the application, which is to send the save: message to the application's managed object context. Any encountered errors are presented to the user.
- (IBAction)saveAction:(id)sender
{
    NSLog(@"%@:%@ saveAction called", [self class], NSStringFromSelector(_cmd));
    NSError *error = nil;
    
    if (![[[OSxCDStackManager sharedManager] managedObjectContext] commitEditing]) {
        NSLog(@"%@:%@ unable to commit editing before saving", [self class], NSStringFromSelector(_cmd));
    }
    
    if (![[[OSxCDStackManager sharedManager] managedObjectContext] save:&error]) {
        [[NSApplication sharedApplication] presentError:error];
    }
}
// Shares in iCloud or removes from iCloud
- (IBAction)shareAction:(id)sender
{
    FLOG(@" called");
    
    NSMenuItem *menuItem = (NSMenuItem *)sender;
    if (_currentContentViewController) {
        [[_currentContentViewController view] removeFromSuperview];
    }
    _currentContentViewController = nil;
    
    if ([menuItem.title isEqualToString:@"Remove from iCloud"]) {
        NSLog(@" remove from iCloud");
        [self performSelectorOnMainThread:@selector(setICloudNo) withObject:nil waitUntilDone:NO];        
    } else {
        NSLog(@" share in iCloud");
        [self performSelectorOnMainThread:@selector(setICloudYes) withObject:nil waitUntilDone:NO];
    }

    FLOG(@" finished");
    
}
- (void)setICloudYes {
    [[OSxCDStackManager sharedManager] setICloudPreference:YES];
}
- (void)setICloudNo {
    [[OSxCDStackManager sharedManager] setICloudPreference:NO];
}

- (IBAction)loadAction:(id)sender
{
    [[OSxCDStackManager sharedManager] loadDataInBackground];
}
- (IBAction)deleteAction:(id)sender
{
    [[OSxCDStackManager sharedManager] deleteDataInBackground];
}
- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender
{
    // Save changes in the application's managed object context before the application terminates.
    
    if (![[OSxCDStackManager sharedManager] managedObjectContext]) {
        return NSTerminateNow;
    }
    
    if (![[[OSxCDStackManager sharedManager] managedObjectContext] commitEditing]) {
        NSLog(@"%@:%@ unable to commit editing to terminate", [self class], NSStringFromSelector(_cmd));
        return NSTerminateCancel;
    }
    
    if (![[[OSxCDStackManager sharedManager] managedObjectContext] hasChanges]) {
        return NSTerminateNow;
    }
    
    NSError *error = nil;
    if (![[[OSxCDStackManager sharedManager] managedObjectContext] save:&error]) {

        // Customize this code block to include application-specific recovery steps.              
        BOOL result = [sender presentError:error];
        if (result) {
            return NSTerminateCancel;
        }

        NSString *question = NSLocalizedString(@"Could not save changes while quitting. Quit anyway?", @"Quit without saves error question message");
        NSString *info = NSLocalizedString(@"Quitting now will lose any changes you have made since the last successful save", @"Quit without saves error question info");
        NSString *quitButton = NSLocalizedString(@"Quit anyway", @"Quit anyway button title");
        NSString *cancelButton = NSLocalizedString(@"Cancel", @"Cancel button title");
        NSAlert *alert = [[NSAlert alloc] init];
        [alert setMessageText:question];
        [alert setInformativeText:info];
        [alert addButtonWithTitle:quitButton];
        [alert addButtonWithTitle:cancelButton];

        NSInteger answer = [alert runModal];
        
        if (answer == NSAlertAlternateReturn) {
            return NSTerminateCancel;
        }
    }

    return NSTerminateNow;
}
- (BOOL)validateUserInterfaceItem:(id < NSValidatedUserInterfaceItem >)anItem {
    NSLog(@" validateUserInterfaceItem: called for item %ld, %@", anItem.tag, anItem);
    
    // Check if the document is already in iCloud
    if ([anItem tag] == 1001) { // iCloud menu
        NSMenuItem *menuItem = (NSMenuItem *)anItem;
        if ([[OSxCDStackManager sharedManager] isCloudEnabled]) {
            [menuItem setTitle:@"Remove from iCloud"];
            return YES;
        }
        else {
            [menuItem setTitle:@"Share in iCloud"];
            return YES;
            
        }
    }
    if ([anItem tag] == 101) { // save menu
        if ([[[OSxCDStackManager sharedManager] managedObjectContext] hasChanges])
            return YES;
        else
            return NO;
    }
    if ([anItem tag] == 102) { // save menu
        if ([[OSxCDStackManager sharedManager] managedObjectContext])
            return YES;
        else
            return NO;
    }
    if ([anItem tag] == 103) { // save menu
        if ([[OSxCDStackManager sharedManager] managedObjectContext])
            return YES;
        else
            return NO;
    }
    return NO;
}

@end
